#!/usr/bin/env python

__author__ = 'Jeffrey Knockel'
__copyright__ = 'Copyright 2013 Jeffrey Knockel'
__license__ = 'GPL v2'

import array
from contextlib import closing
try:
    from fcntl import ioctl
except ImportError:
    ioctl = None
import socket
import struct
import sys

def max_iface_mtu(using_iface=None):
    mtus = []
    if sys.platform.startswith('linux'):
        SIOCGIFCONF = 0x8912
        SIOCGIFFLAGS = 0x8913
        SIOCGIFMTU = 0x8921
        IFF_UP = 0x1
        IFF_LOOPBACK = 0x8
        IFF_RUNNING = 0x40
        with closing(socket.socket(socket.AF_INET, socket.SOCK_DGRAM, socket.IPPROTO_IP)) as s:
            ary = array.array('B', '\0' * (128 * 32))
            n = struct.unpack('iP', ioctl(s, SIOCGIFCONF, struct.pack('iP', 128 * 32, ary.buffer_info()[0])))[0]
            buf = ary.tostring()
            ifaces = []
            for i in xrange(0, n, 40):
                ifaces.append(buf[i:i + 16].split('\0', 1)[0])
            for iface in ifaces:
                iface, flags = struct.unpack('=16sH', ioctl(s, SIOCGIFFLAGS, struct.pack('=16sH', iface, 0)))
                iface = iface.split('\0')[0]
                if (using_iface is not None and iface == using_iface) or \
                        (using_iface is None and flags & IFF_UP and flags & IFF_RUNNING and not flags & IFF_LOOPBACK):
                    iface, mtu = struct.unpack('=16si', ioctl(s, SIOCGIFMTU, struct.pack('=16si', iface, 0)))
                    mtus.append(mtu)
    return max(mtus) if mtus else None

def main():
    mtu = max_iface_mtu()
    print mtu

if __name__ == '__main__':
    main()
