#!/usr/bin/env python

__author__ = 'Jeffrey Knockel'
__copyright__ = 'Copyright 2013 Jeffrey Knockel'
__license__ = 'GPL v2'

import sys

from icmp6 import send_icmp, receive_icmp
from ifacemtu6 import max_iface_mtu
from intervention import intervention_at_time
from nslookup6 import lookup_or_die
from scanerror import ScanError
from sequence import ICMP6Sequence
from sniffing import start_sniffing6 as start_sniffing

INTERVENTION = 4
SAMPLES = 20
TIMEOUT = 1.0

def test_mtu(sniffer, addr1, addr2, mtu, iface_mtu):
    iface_mtu = (iface_mtu + 8) // 8 * 8
    last_ip_id = None
    xs = []
    seq = ICMP6Sequence(addr1, iface=sniffer.iface)
    seq_spoofed = ICMP6Sequence(addr1, addr2)
    timeout = sniffer.timeout()
    for i in xrange(SAMPLES + 1):
        timeout.add_time(TIMEOUT)
        if i > SAMPLES // 2:
            for _ in xrange(INTERVENTION):
                spoofed = seq_spoofed.next_packet_testing_mtu(mtu)
                send_icmp(sniffer, spoofed)
        request = seq.next_packet_testing_mtu(iface_mtu)
        send_icmp(sniffer, request)
        sniffer.work()
        p = receive_icmp(sniffer, request, timeout, True)
        frag = None if p is None else p.extension_hdrs[44]
        ip_id = None if frag is None else frag.id
        if i > 0:
            diff = None if ip_id is None or last_ip_id is None else ip_id - last_ip_id - 1 & 0xffffffff
            sys.stderr.write('%s%s: %s\n' % (ip_id, '*' * (i > SAMPLES // 2), diff))
            xs.append(diff)
        last_ip_id = ip_id
        timeout.sleep()
    if all(x is None for x in xs):
        raise ScanError('Cannot get \'from\' host\'s IP id differences--does it respond to large ICMP echo requests?')
    try:
        return intervention_at_time(xs, [SAMPLES // 2], [INTERVENTION / 2.0], 0.05, True)
    except Exception:
        raise ScanError('The \'from\' host\'s traffic too noisy or incomplete--try another host?')

def usage(code=1):
    sys.stderr.write('Usage: %s HOST1 HOST2 MTU\n' % sys.argv[0])
    exit(code)

def main():
    try:
        if len(sys.argv) != 4:
            usage()
        host1 = sys.argv[1]
        host2 = sys.argv[2]
        try:
            mtu = int(sys.argv[3])
        except ValueError:
            usage()
        addr1 = lookup_or_die(host1)[0]
        addr2 = lookup_or_die(host2)[0]
        ir = start_sniffing(test_mtu, addr1, addr2, mtu, max_iface_mtu())
        print ir
    except ScanError as e:
        sys.stderr.write('Error performing scan: %s\n' % e)
    except KeyboardInterrupt:
        sys.stderr.write('Received SIGINT\n')

if __name__ == '__main__':
    main()
