#!/usr/bin/env python

__author__ = 'Jeffrey Knockel'
__copyright__ = 'Copyright 2013 Jeffrey Knockel'
__license__ = 'GPL v2'

import socket
import sys

def lookup(host):
    families, socktypes, protos, canonnames, sockaddrs = zip(*socket.getaddrinfo(host, None, socket.AF_INET6, socket.SOCK_STREAM, socket.SOL_TCP))
    addrs, ports, flow_infos, scope_ids = zip(*sockaddrs)
    return addrs

def lookup_or_die(host, code=1):
    try:
        return lookup(host)
    except socket.gaierror as e:
        sys.stderr.write('%s: %s\n' % (host, e.strerror))
        exit(code)

def usage(code=1):
    sys.stderr.write('Usage: %s HOST\n' % sys.argv[0])
    exit(code)

def main():
    if len(sys.argv) != 2:
        usage()
    host = sys.argv[1]
    addrs = lookup_or_die(host)
    for addr in addrs:
        print(addr)

if __name__ == '__main__':
    main()
